/* From a340dack@cdf.toronto.edu Wed Oct 13 12:33:33 1999 */

/* pushbroom.c */

#include <stdlib.h>
#include <stdio.h>

#include <asm/types.h>   /* headers for architecture specific data types        
                 */
#include <unistd.h>      /* contains the prototype for i386 specific ioperm func
tioni            */
#include <asm/io.h>      /* obviously architecture specific (asm directory) for 
inb, outb macros */

#define USING_PARALLEL

#ifdef USING_PARALLEL
#   define LP_BASE 0x278
#   define PRT_CHANGE   (__u8)
#else
#   define LP_BASE 0x240
#   define PRT_CHANGE ~ (__u8)
#endif

#define DUN                       0

#define BEGIN_NUMERIC '0'
#define END_NUMERIC   '9'
#define BEGIN_ALPHA       'A'
#define END_ALPHA         'Z'

#define MAX_RECYCLE   3
#define SPACE_GAP     55000
#define RECYCLE_GAP   4

__u8 gaAlpha[][5] = 
{
/*A*/   {252,18 ,17,  18,252}, 
/*B*/   {255,137,137,137,118},
/*C*/   {126,129,129,129, 66},
/*D*/   {255,129,129, 66, 60},
/*E*/   {255,137,137,129,DUN},
/*F*/   {255,  9,  9,  1,DUN},
/*G*/   {126,129,145, 81,242},
/*H*/   {255, 16, 16, 16,255},
/*I*/   {129,255,129,DUN,DUN},
/*J*/   { 64,128,129,127,  1},
/*K*/   {255, 24, 36, 66,129},
/*L*/   {255,128,128,128,DUN},
/*M*/   {255,  2, 12,  2,255},
/*N*/   {255,  6, 24, 96,255},
/*O*/   {126,129,129,129,126},
/*P*/   {255, 17, 17, 17, 14},
/*Q*/   {126,129,161, 65,190},
/*R*/   {255, 17, 17, 49,206},
/*S*/   { 78,137,137,137,114},
/*T*/   {  1,  1,255,  1,  1},
/*U*/   {127,128,128,128,127},
/*V*/   { 31, 96,128, 96, 31},
/*W*/   { 63,192, 56,192, 63},
/*X*/   {195, 36, 24, 36,195},
/*Y*/   {  7,  8,240,  8,  7},
/*Z*/   {225,145,137,133,131}
};

__u8 gaNumeric[][5] = 
{
/*0*/   {126,135,153,255,126},
/*1*/   {130,255,128,DUN,DUN},
/*2*/   {198,161,145,137,134},
/*3*/   { 66,129,137,137,118},
/*4*/   { 56, 36, 34,255, 32},
/*5*/   { 79,137,137,137,113},
/*6*/   {126,137,137,137,113},
/*7*/   {  1,  1,241, 13,  3},
/*8*/   {118,137,137,137,118},
/*9*/   { 14, 17, 17, 17,255}
};

int main(int argc, char *argv[])
{
        int i;
        int iRecycleCount = 0;
        int iStringIndex  = 0;
        int iCharIndex    = 0;

        /*Gimme args!!!! */
        if (argc < 2)
        {
                fprintf(stderr,"Enter a single string as a parameter!");
                return 1;
        }


        /* attempt to reserve region in io space */
        if(ioperm(LP_BASE, 3, 1) < 0) {
                fprintf(stderr, "Cannot reserve region in io space, aborting...\n");
                exit(1);
        }

        for(;;)
        {
                /* Hit a NULL? --recycle-- */
                if (argv[1][iStringIndex] == 0)
                {
                        /* push a 0 out and wait a bit to recycle */
                        outb(PRT_CHANGE 0, LP_BASE);

                        /* only recycle a finite number of times */
                        if (++iRecycleCount > MAX_RECYCLE)
                                break;

                        sleep(RECYCLE_GAP);
                        iStringIndex = 0;
                }

                /* uppercase the chars... */
                argv[1][iStringIndex] = toupper(argv[1][iStringIndex]);

                /* ALPHA */
                if (argv[1][iStringIndex] >= BEGIN_ALPHA && 
                        argv[1][iStringIndex] <= END_ALPHA     )
                {
                        iCharIndex = argv[1][iStringIndex] - BEGIN_ALPHA;

                        for (i=0;i < 5; i++)
                        {
                                if (gaAlpha[iCharIndex][i] == DUN )
                                        break;

                                /* push out gaAlpha[iCharIndex][i] */
                                outb(PRT_CHANGE gaAlpha[iCharIndex][i], LP_BASE);
                                usleep(SPACE_GAP);
                        }
                }

                /* NUMERIC */
                else if (argv[1][iStringIndex] >= BEGIN_NUMERIC && 
                        argv[1][iStringIndex] <= END_NUMERIC     )
                {
                        iCharIndex = argv[1][iStringIndex] - BEGIN_NUMERIC;

                        for (i=0; i < 5; i++)
                        {
                                if (gaNumeric[iCharIndex][i] == DUN)
                                        break;

                                /* push out gaNumeric[iCharIndex][i] */
                                outb(PRT_CHANGE gaNumeric[iCharIndex][i], LP_BASE);
                                usleep(SPACE_GAP);
                        }
                }

                /* otherwise assume space */
                else 
                {
                        /* push out 0  and wait for five sleeps */
                        outb(PRT_CHANGE 0, LP_BASE);
                        usleep(5*SPACE_GAP);
                }

                /* intra-character space */
                /* push out 0 and wait for two sleeps*/
                outb(PRT_CHANGE 0, LP_BASE);
                usleep(2*SPACE_GAP);

                /*increment to next char */
                iStringIndex++;
        }

        /* attempt to release reserved region in io space */
        if(ioperm(LP_BASE, 3, 0) < 0) {
                fprintf(stderr, "Cannot release region in io space, aborting...\n");
                exit(1);
        }

        return 0;
}

