/*
 * Thanks to Scot Laird, Russell Nelson, and others..
 *
 * Copyright (C) 1996 by Thomas Davis
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL ANY OF THE COPYRIGHT HOLDERS BE HELD LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <assert.h>
#include <time.h>
#include "quickcam.h"

#define FRAME_SIZE 160*120

void main()
{
	int fd;
	int ret;
	int count;
	unsigned char buf[FRAME_SIZE];
	float rate;
   
	int num=50;
	long startTime;
	long endTime;
	long diff;
	int i;
	struct qcam qc;
   
	fprintf(stderr, "performance test started\n");
   

	fd = open( "/dev/quickcam" , O_RDONLY );

	if ( fd < 0 ) {
		fprintf(stderr,"performance open failed with %d \n",fd);
		exit(1);
	}
     
	fprintf(stderr, "6bpp, 160x120, bi-directional mode\n");

	startTime = time(NULL);
   
	qc.xsize = 160;
	qc.ysize = 120;
	qc.bpp = 6;
	qc.mode = QC_BIDIR;
	qc.brightness = 120;
	qc.whitebalance = 127;
	qc.contrast = 120;
	qc.debug = 0;

	ret = ioctl(fd, QC_SET, &qc);

	if (ret != 0) {
		fprintf(stderr, "ioctl problems dear...\n");
		exit(-1);
	}
	
	count = FRAME_SIZE;
	for(i = 0; i < num; i++ ) {
#if 0
		fprintf(stderr,"try %d \n",i);
#endif
		lseek(fd, 0, SEEK_SET);
		ret = read(fd, buf, count);
		assert( ret == FRAME_SIZE );
	}

	endTime = time(NULL);
   
	diff = endTime-startTime;
	fprintf(stderr,"read in %ld seconds \n",diff);
   
	rate = (float)(num)/(float)(diff);
	fprintf(stderr,"transfer rate %3.1f frames per second\n",rate);

	rate = (float)(num)*FRAME_SIZE/( (float)(diff)*1024*1024 );
	fprintf(stderr,"transfer rate %3.1f meg. bytes per second\n",rate);

	ret = ioctl(fd, QC_GET, &qc);

	if (ret != 0) {
		fprintf(stderr, "ioctl problems dear...\n");
		exit(-1);
	}

	fprintf(stderr, "kernel reports %d frames generated..\n", qc.framecount);

	fprintf(stderr, "4bpp, 160x120, uni-directional..\n");

	startTime = time(NULL);
   
	qc.bpp = 4;
	qc.mode = QC_UNIDIR;
	qc.debug = 0;
   
	ret = ioctl(fd, QC_SET, &qc);
#if DEBUG
	fprintf(stderr, "ret = %d\n", ret);
#endif

	if (ret != 0) {
		fprintf(stderr, "ioctl problems dear...\n");
		exit(-1);
	}
	
	count = FRAME_SIZE;
	for(i = 0; i < num; i++ ) {
#if 0
		fprintf(stderr,"try %d \n",i);
#endif
		lseek(fd, 0, SEEK_SET);
		ret = read(fd, buf, count);
		assert( ret == FRAME_SIZE );
	}

	endTime = time(NULL);
   
	diff = endTime-startTime;
	fprintf(stderr,"read in %ld seconds \n",diff);
   
	rate = (float)(num)/(float)(diff);
	fprintf(stderr,"transfer rate %3.1f frames per second\n",rate);

	rate = (float)(num)*FRAME_SIZE/( (float)(diff)*1024*1024 );
	fprintf(stderr,"transfer rate %3.1f meg. bytes per second\n",rate);

	ret = ioctl(fd, QC_GET, &qc);

	if (ret != 0) {
		fprintf(stderr, "ioctl problems dear...\n");
		exit(-1);
	}

	fprintf(stderr, "kernel reports %d frames generated..\n", qc.framecount);

	fprintf(stderr, "6bpp, 160x120, uni-directional..\n");

	startTime = time(NULL);
   
	qc.bpp = 6;
	qc.mode = QC_UNIDIR;
   
	ret = ioctl(fd, QC_SET, &qc);
#if DEBUG
	fprintf(stderr, "ret = %d\n", ret);
#endif

	if (ret != 0) {
		fprintf(stderr, "ioctl problems dear...\n");
		exit(-1);
	}
	
	count = FRAME_SIZE;
	for(i = 0; i < num; i++ ) {
#if 0
		fprintf(stderr,"try %d \n",i);
#endif
		lseek(fd, 0, SEEK_SET);
		ret = read(fd, buf, count);
		assert( ret == FRAME_SIZE );
	}

	endTime = time(NULL);
   
	diff = endTime-startTime;
	fprintf(stderr,"read in %ld seconds \n",diff);
   
	rate = (float)(num)/(float)(diff);
	fprintf(stderr,"transfer rate %3.1f frames per second\n",rate);

	rate = (float)(num)*FRAME_SIZE/( (float)(diff)*1024*1024 );
	fprintf(stderr,"transfer rate %3.1f meg. bytes per second\n",rate);

	ret = ioctl(fd, QC_GET, &qc);

	if (ret != 0) {
		fprintf(stderr, "ioctl problems dear...\n");
		exit(-1);
	}

	fprintf(stderr, "kernel reports %d frames generated..\n", qc.framecount);

	close(fd);

	startTime = time(NULL);
   
	count = FRAME_SIZE;
	for( i=0 ; i <num ; i++ ) {
#if 0
		fprintf(stderr,"try %d \n",i);
#endif
		fd = open( "/dev/quickcam" , O_RDONLY );
		ret=read(fd, buf, count);
		assert( ret == FRAME_SIZE );
		close(fd);
	}   
	endTime = time(NULL);
   
	diff = endTime-startTime;
	fprintf(stderr,"read in %ld seconds \n",diff);
   
	rate = (float)(num)/(float)(diff);
	fprintf(stderr,"grab rate %3.1f frames per second\n",rate);

	rate = (float)(num)*FRAME_SIZE/( (float)(diff)*1024*1024 );
	fprintf(stderr,"grab rate %3.1f meg. bytes per second\n",rate);
	fprintf(stderr,"performance done\n");

	close(fd);

	exit(0);
}
