/*
 * Thanks to Scot Laird, Russell Nelson, and others..
 *
 * Copyright (C) 1996 by Thomas Davis
 *
 * Copyright (C) 1996 by Scott Laird
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL SCOTT LAIRD BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */


#ifndef _QCAM_KERNEL_H

#define _QCAM_KERNEL_H 1

/*
 * these are private to the driver..
 */

#define QC_BRIGHTNESS	0x0b
#define QC_CONTRAST	0x19
#define QC_WHITEBALANCE	0x1f
#define QC_XFERMODE	0x07
#define QC_XSIZE	0x13
#define QC_YSIZE	0x11
#define QC_UNK1		0x0d
#define QC_UNK2		0x0f

/*
 * kernel structure..
 */

struct quickcam {
	int xsize;			/* x size */
	int ysize;			/* y size */
	unsigned int port;		/* port address */
	unsigned char *buffer;		/* video buffer */
	char bpp;			/* bits-per-pixel */
	char mode;			/* transfer mode */
	unsigned char status;		/* port status (uni/bi) */
	unsigned char xfermode;		/* xfermode */
	unsigned char brightness;	/* brightness */
	unsigned char whitebalance;	/* whitebalance */
	unsigned char contrast;		/* contrast */
	unsigned char panx;		/* */
	unsigned char pany;		/* */
	unsigned char zoom;		/* zoom mode.. */
	char init;			/* send the values to the camera */
	int debug;			/* debug values. */
	int count;			/* if <= 0, camera problems */
	int framecount;		/* frames grabbed at current settings */
	int starttime;		/* jiffy count at start of capture */
	int endtime;		/* jiffy count at end of capture */
	int avgtime;		/* average of ((endtime-starttime) + avgtime)/2 */
};

/*
 * prototypes for the kernel driver.
 */

extern void uni_4bit(struct quickcam *);
extern void uni_6bit(struct quickcam *);
extern void bi_4bit(struct quickcam *);
extern void bi_6bit(struct quickcam *);
extern void reset_quickcam(struct quickcam *);
extern int waitfor_bi(struct quickcam *);
extern int doscan(struct quickcam *);
extern int qcam_init(struct quickcam *);
extern int qcam_find(int);

#endif
